@@ -10,6 +10,11 @@ from pyravendb.data.timeseries import TimeSeriesRangeResult
 from .session_timeseries import TimeSeries
 from .session_counters import DocumentCounters
 from typing import Dict, List
+try:
+    from collections.abc import MutableSet
+except ImportError:
+    from collections import MutableSet
+from itertools import chain
 
 
 class _SaveChangesData(object):
@@ -19,6 +24,139 @@ class _SaveChangesData(object):
         self.deferred_command_count = deferred_command_count
 
 
+class _RefEq:
+    def __init__(self, ref):
+        if isinstance(ref, _RefEq):
+            self.ref = ref.ref
+            return
+        self.ref = ref
+
+    # As we split the hashable and unhashable items into separate collections, we only compare _RefEq to other _RefEq
+    def __eq__(self, other):
+        if isinstance(other, _RefEq):
+            return id(self.ref) == id(other.ref)
+        raise TypeError("Expected _RefEq type object")
+
+    def __hash__(self):
+        return id(self.ref)
+
+
+class _RefEqEntityHolder(object):
+    def __init__(self):
+        self.unhashable_items = dict()
+
+    def __len__(self):
+        return len(self.unhashable_items)
+
+    def __contains__(self, item):
+        return _RefEq(item) in self.unhashable_items
+
+    def __delitem__(self, key):
+        del self.unhashable_items[_RefEq(key)]
+
+    def __setitem__(self, key, value):
+        self.unhashable_items[_RefEq(key)] = value
+
+    def __getitem__(self, key):
+        return self.unhashable_items[_RefEq(key)]
+
+    def __getattribute__(self, item):
+        if item == "unhashable_items":
+            return super().__getattribute__(item)
+        return self.unhashable_items.__getattribute__(item)
+
+
+class _DocumentsByEntityHolder(object):
+    def __init__(self):
+        self._hashable_items = dict()
+        self._unhashable_items = _RefEqEntityHolder()
+
+    def __repr__(self):
+        return f"{self.__class__.__name__}: {[item for item in self.__iter__()]}"
+
+    def __len__(self):
+        return len(self._hashable_items) + len(self._unhashable_items)
+
+    def __contains__(self, item):
+        try:
+            return item in self._hashable_items
+        except TypeError as e:
+            if str(e.args[0]).startswith("unhashable type"):
+                return item in self._unhashable_items
+            raise e
+
+    def __setitem__(self, key, value):
+        try:
+            self._hashable_items[key] = value
+        except TypeError as e:
+            if str(e.args[0]).startswith("unhashable type"):
+                self._unhashable_items[key] = value
+                return
+            raise e
+
+    def __getitem__(self, key):
+        try:
+            return self._hashable_items[key]
+        except (TypeError, KeyError):
+            return self._unhashable_items[key]
+
+    def __iter__(self):
+        d = list(map(lambda x: x.ref, self._unhashable_items.keys()))
+        if len(self._hashable_items) > 0:
+            d.extend(self._hashable_items.keys())
+        return (item for item in d)
+
+    def get(self, key, default=None):
+        return self[key] if key in self else default
+
+    def pop(self, key, default_value=None):
+        result = self._hashable_items.pop(key, None)
+        if result is not None:
+            return result
+        return self._unhashable_items.pop(_RefEq(key), default_value)
+
+    def clear(self):
+        self._hashable_items.clear()
+        self._unhashable_items.clear()
+
+    def items(self):
+        return list(
+            chain(
+                map(lambda item: (item[0], item[1]), self._hashable_items.items()),
+                map(
+                    lambda item: (item[0].ref, item[1]), self._unhashable_items.items()
+                ),
+            )
+        )
+
+
+class _DeletedEntitiesHolder(MutableSet):
+    def __init__(self, items=None):
+        if items is None:
+            items = []
+        self.items = set(map(_RefEq, items))
+
+    def __getattribute__(self, item):
+        if item in ["add", "discard", "items"]:
+            return super().__getattribute__(item)
+        return self.items.__getattribute__(item)
+
+    def __contains__(self, item: object) -> bool:
+        return _RefEq(item) in self.items
+
+    def __len__(self) -> int:
+        return len(self.items)
+
+    def __iter__(self):
+        return (item.ref for item in self.items)
+
+    def add(self, element: object) -> None:
+        return self.items.add(_RefEq(element))
+
+    def discard(self, element: object) -> None:
+        return self.items.discard(_RefEq(element))
+
+
 class DocumentSession(object):
     def __init__(self, database, document_store, requests_executor, session_id, **kwargs):
         """
@@ -33,8 +171,8 @@ class DocumentSession(object):
         self._requests_executor = requests_executor
         self._documents_by_id = {}
         self._included_documents_by_id = {}
-        self._deleted_entities = set()
-        self._documents_by_entity = {}
+        self._deleted_entities = _DeletedEntitiesHolder()
+        self._documents_by_entity = _DocumentsByEntityHolder()
         self._timeseries_defer_commands = {}
         self._time_series_by_document_id = {}
         self._counters_defer_commands = {}
@@ -48,6 +186,19 @@ class DocumentSession(object):
         self.no_tracking = kwargs.get("no_tracking", False)
         self.no_caching = kwargs.get("no_caching", False)
         self.advanced = Advanced(self)
+        self._events = None
+
+    @property
+    def readonly_events(self):
+        if self._events is None:
+            return self._document_store.events.clone()
+        return self._events
+
+    @property
+    def events(self):
+        if self._events is None:
+            self._events = self._document_store.events.clone()
+        return self._events
 
     def __enter__(self):
         return self
@@ -141,6 +292,7 @@ class DocumentSession(object):
         if key not in self._documents_by_id:
             entity, metadata, original_metadata, original_document = Utils.convert_to_entity(document, object_type,
                                                                                              self.conventions,
+                                                                                             self.readonly_events,
                                                                                              nested_object_types)
             self.save_entity(key, entity, original_metadata, metadata, original_document)
 
@@ -235,6 +387,7 @@ class DocumentSession(object):
         if "Raven-Read-Only" in self._documents_by_entity[entity]["original_metadata"]:
             raise exceptions.InvalidOperationException(
                 "{0} is marked as read only and cannot be deleted".format(entity))
+
         self._included_documents_by_id.pop(self._documents_by_entity[entity]["key"], None)
         self._known_missing_ids.add(self._documents_by_entity[entity]["key"])
         self._deleted_entities.add(entity)
@@ -375,6 +528,8 @@ class DocumentSession(object):
                         document_info["metadata"] = item
                         document_info["key"] = item["@id"]
                         document_info["original_value"] = entity.__dict__.copy()
+                        self.readonly_events.after_save_change(self, item["@id"], entity)
+
                 elif item["Type"] == "Counters":
                     cache = self._counters_by_document_id.get(item["Id"], None)
                     if not cache:
@@ -415,13 +570,15 @@ class DocumentSession(object):
                 change_vector = None
                 if self.advanced.use_optimistic_concurrency and self._documents_by_entity[entity][
                     "force_concurrency_check"] != "disabled" or self._documents_by_entity[entity][
-                    "force_concurrency_check"] == "forced":
+                        "force_concurrency_check"] == "forced":
                     change_vector = self._documents_by_entity[entity]["change_vector"]
+                self.readonly_events.before_store(self, key, entity)
                 data.entities.append(entity)
                 if key:
                     self._documents_by_id.pop(key)
                     document = entity.__dict__.copy()
                     document.pop('Id', None)
+
                 data.commands.append(commands_data.PutCommandData(key, change_vector, document, metadata))
 
     def _has_change(self, entity):
@@ -460,6 +617,9 @@ class Advanced(object):
             self._attachment = _Attachment(self.session)
         return self._attachment
 
+    def get_metadata_for(self, entity):
+        return self.session.documents_by_entity[entity]['metadata']
+
     def stream(self, query):
         from pyravendb.store.stream import IncrementalJsonParser
         import ijson
@@ -477,6 +637,7 @@ class Advanced(object):
             results = ijson.backend.common.items(parser, "Results")
             for result in next(results, None):
                 document, metadata, _, _ = Utils.convert_to_entity(result, query.object_type, self.session.conventions,
+                                                                   self.session.readonly_events,
                                                                    query.nested_object_types)
                 yield {"document": document, "metadata": metadata, "id": metadata.get("@id", None),
                        "change-vector": metadata.get("@change-vector", None)}
